// ------------------------------------------------------------------------------------------------
// Modeling Transformations: Rotate, Scale, Translate
// The Modelview matrix
// Example of composing transformations
// A polygon rotates about its axis (spin)

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
for M3*M2*M1*P in OpenGL we have postmultiplying, namely from right-to-left
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

// Modeling transformations: 

/* Commands: glPushMatrix(), glPopMatrix(), for saving and restoring current matrix
	in Modelview matrix stack 
*/
		
/* Commands:
	glRotate{fd}(TYPE angle,TYPE x,TYPE y,TYPE z) for rotating
	glScale{fd}(TYPE x, TYPE y, TYPE z) for stretching, shrinking, reflecting 
	glTranslate{fd}(TYPE x, TYPE y, TYPE z), for translating

	All three commands are equivalent to producing an appropriate translation, rotation, or 
	scaling matrix, and then calling glMultMatrix*() with your matrix as the argument.
	Using these three routines might be faster. OpenGL automatically computes the matrices for you.
*/
//------------------------------------------------------------------------------------------------------


#include <glut.h> 

GLint width=640;
GLint height=480;

void myInit(void)
{
	glClearColor(0.5, 0.5, 0.5, 0.0); // background is gray (Black backgound may be caused by other reasons)
							 
	glMatrixMode(GL_PROJECTION); // update projection
	glLoadIdentity();
	gluOrtho2D(0.0, (GLdouble)width, 0.0, (GLdouble)height); //coordinates are floating ponts - 
															// gluOrtho2D follows OpenGL convention.
				//Try dividing w and h by 10 !!!!!!!!!!!
}

void myReshape(int w, int h) // window is reshaped 
{
	glViewport(0,0,w,h); //// coordinates are in pixels - vieport follows glut convention.
								  // by default viewport is equal to display winow	
	//glViewport(width/2,0,width,height); 
	//glViewport(width/2,0,width/2,height);
	glMatrixMode(GL_PROJECTION); // update projection
	glLoadIdentity();
	gluOrtho2D(0.0, (GLdouble)w, 0.0, (GLdouble)h); //Try dividing w and h by 10 !!!!!!!!!!!
	glMatrixMode(GL_MODELVIEW);// GL_MODELVIEW is the default argument for glMatrixMode(),
								// so we turn to it after glMatrixMode(GL_PROJECTION)
	glutPostRedisplay();        // request redisplay
}

void myDisplay(void) // (re)display callback
{
	
	glClear(GL_COLOR_BUFFER_BIT);    // clear the window

	glColor3f(1.0, 0.0, 0.0);
	glRectf(25.0, 25.0, 75.0, 75.0);

	glColor3f(0.0, 1.0, 0.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glPushMatrix();
	glTranslatef(50.0,50.0,0.0); // this matrix is the third to be applied	
								// we translate the polygon at its initial position
	glRotatef(45,0,0,1);		// this matrix is the second to be applied 
								// glRotatef rotates the object about the origin(0,0)
	glTranslatef(-50.0,-50.0,0.0); // this matrix is the first to be applied,
								// it translates the center of the polygon to
								// the origin(0.0)

	glBegin(GL_POLYGON);        // draw the polygon
		glVertex2f(25.0, 25.0);
		glVertex2f(75.0, 25.0);
		glVertex2f(75.0, 75.0);
		glVertex2f(25.0, 75.0);
	glEnd();
	
	glPopMatrix();

	glutSwapBuffers(); // When we have (GLUT_DOUBLE)we draw in the back buffer 
						// and this command brings it in front
}


int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB); // GLUT_DOUBLE always goes with glutSwapBuffers();
	glutInitWindowSize(width, height);
	glutInitWindowPosition(100,100);
	glutCreateWindow("modelTransformations");
	glutDisplayFunc(myDisplay);
	glutReshapeFunc(myReshape);
	
	myInit();
	
	glutMainLoop();
	return 0;
}
